/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.isobus.project;

import java.awt.Image;
import java.awt.datatransfer.Transferable;
import javax.swing.Action;
import org.netbeans.spi.project.ui.LogicalViewProvider;
import org.netbeans.spi.project.ui.support.CommonProjectActions;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataFolder;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.FilterNode;
import org.openide.nodes.Node;
import org.openide.util.Exceptions;
import org.openide.util.HelpCtx;
import org.openide.util.ImageUtilities;
import org.openide.util.Lookup;
import org.openide.util.datatransfer.PasteType;
import org.openide.util.lookup.Lookups;
import org.openide.util.lookup.ProxyLookup;

/**
 * Provides a logical view (Isobus project node-structure) in the explorer-tab.
 *
 * @author Gerben Feenstra
 */
public class IsobusLogicalView implements LogicalViewProvider {

    private static final String PROJECT_ICON = "nl/cloudfarming/client/isobus/project/isobus_project.png";
    private final IsobusProject project;

    public IsobusLogicalView(IsobusProject project) {
        this.project = project;
    }

    /**
     * Create a root node based on the projects' folder
     *
     * @return Root node
     */
    @Override
    public Node createLogicalView() {
        try {
            FileObject projectFolder = project.getProjectDirectory();
            DataFolder projectDataFolder = DataFolder.findFolder(projectFolder);
            
            
            FileObject taskfilesFolder = project.getTaskfilesFolder(true);
            DataFolder taskfilesDataFolder = DataFolder.findFolder(taskfilesFolder);
            
            //This FilterNode will be our root project node
            return new IsobusProjectFolderNode(projectDataFolder.getNodeDelegate(), taskfilesDataFolder.getNodeDelegate(), project);

        } catch (DataObjectNotFoundException donfe) {
            Exceptions.printStackTrace(donfe);
            //Fallback-the directory couldn't be created -
            //read-only filesystem or something evil happened
            return new AbstractNode(Children.LEAF);
        }
    }

    @Override
    public Node findPath(Node root, Object target) {
        //leave unimplemented for now
        return null;
    }

    /**
     * The rootnode of the project.
     *
     * This is the node you actually see in the project tab for this project
     */
    private static final class IsobusProjectFolderNode extends FilterNode {

        final IsobusProject project;

        public IsobusProjectFolderNode(Node projectNode, Node taskfilesNode, IsobusProject project) throws DataObjectNotFoundException {
            super(projectNode //                    , new IsobusProjectChildrenFactory(node, project)
                    , new FilterNode.Children(taskfilesNode),
                    //The projects system wants the project in the Node's lookup.
                    //NewAction and friends want the original Node's lookup.
                    //Make a merge of both
                    new ProxyLookup(new Lookup[]{Lookups.singleton(project),
                        projectNode.getLookup()
                    }));
            this.project = project;
        }

        /**
         * Standard project Actions + import isobus action
         *
         * @param context
         * @return
         */
        @Override
        public Action[] getActions(boolean context) {
            return new Action[]{
                        CommonProjectActions.newFileAction(),
                        CommonProjectActions.copyProjectAction(),
                        CommonProjectActions.deleteProjectAction(),
                        CommonProjectActions.setAsMainProjectAction(),
                        CommonProjectActions.closeProjectAction(),
                        new ImportIsobusFiles(project)
                    };
        }

        @Override
        public Image getIcon(int type) {
            return ImageUtilities.loadImage(PROJECT_ICON);
        }

        @Override
        public Image getOpenedIcon(int type) {
            return getIcon(type);
        }

        @Override
        public String getDisplayName() {
            return project.getProjectDirectory().getName();
        }

        @Override
        public String getHtmlDisplayName() {
            return project.getProjectDirectory().getName();
        }
        
        @Override
        public HelpCtx getHelpCtx() {
            return new HelpCtx("nl.cloudfarming.client.isobus.project.explorer");
        }

        @Override
        public PasteType getDropType(Transferable t, int action, int index) {
            return null;
        }
    }
}
